# -*- coding: utf-8 -*-
#
# Copyright 2013, 2014 Develer S.r.l. (https://www.develer.com/)
# Copyright 2018 wyDay, LLC (https://wyday.com/)
#
# Current Author / maintainer:
#
#   Author: wyDay, LLC <support@wyday.com>
#
#
# Previous authors (and based on their fantastic work):
#
#   Author: Lorenzo Villani <lvillani@develer.com>
#   Author: Riccardo Ferrazzo <rferrazz@develer.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

from ctypes import pointer, sizeof, c_uint32

from turboactivate.c_wrapper import *

import os
import sys

#
# Object oriented interface
#

class IsGenuineResult:
    Genuine, GenuineFeaturesChanged, NotGenuine, NotGenuineInVM, InternetError = range(5)


class TurboActivate(object):

    def __init__(self, guid, flags = TA_USER, dat_file_loc = "", library_folder = ""):

        # load the executing file's location
        if getattr(sys, 'frozen', False):
            # running in a bundle
            execFileLoc = os.path.dirname(os.path.abspath(sys.executable))
        else:
            # running live
            execFileLoc = os.path.dirname(os.path.abspath(sys.modules['__main__'].__file__))

        if not library_folder:
            library_folder = execFileLoc

        # form the full, absolute path to the TurboActivate.dat file
        if not dat_file_loc:
            dat_file_loc = os.path.join(execFileLoc, "TurboActivate.dat")

        self._lib = load_library(library_folder)
        self._set_restype()

        self._flags = flags

        try:
            self._lib.TA_PDetsFromPath(wstr(dat_file_loc))
        except TurboActivateFailError:
            # The dat file is already loaded
            pass

        self._handle = self._lib.TA_GetHandle(wstr(guid))

        # if the handle is still unset then immediately throw an exception
        # telling the user that they need to actually load the correct
        # TurboActivate.dat and/or use the correct GUID for the TurboActivate.dat
        if self._handle == 0:
            raise TurboActivateDatFileError()

    #
    # Public
    #

    # Product key

    def check_and_save_pkey(self, product_key):
        """Checks and saves the product key."""
        ret = self._lib.TA_CheckAndSavePKey(self._handle, wstr(product_key), self._flags)

        if ret == TA_OK:
            return True
        elif ret == TA_FAIL:
            return False

        validate_result(ret)

    def is_product_key_valid(self):
        """
        Checks if the product key installed for this product is valid. This does NOT check if
        the product key is activated or genuine. Use is_activated() and is_genuine_ex() instead.
        """
        try:
            self._lib.TA_IsProductKeyValid(self._handle)

            return True
        except TurboActivateError:
            return False

    def get_pkey(self):
        """
        Gets the stored product key. NOTE: if you want to check if a product key is valid
        simply call is_product_key_valid().
        """
        buf_size = 35
        buf = wbuf(buf_size)

        try:
            self._lib.TA_GetPKey(self._handle, buf, buf_size)

            return buf.value
        except TurboActivateProductKeyError:
            return None


    # Activation status

    def deactivate(self, erase_p_key=False):
        """
       Deactivates the product on this computer. Set erasePkey to 1 to erase the stored
       product key, 0 to keep the product key around. If you're using deactivate to let
       a user move between computers it's almost always best to *not* erase the product
       key. This way you can just use TA_Activate() when the user wants to reactivate
       instead of forcing the user to re-enter their product key over-and-over again.
        """

        if erase_p_key is True:
            args = 1
        else:
            args = 0

        self._lib.TA_Deactivate(self._handle, args)

    def deactivation_request_to_file(self, filename, erase_p_key=False):
        """
        Get the "deactivation request" file for offline deactivation. Set erase_p_key to
        True to erase the stored product key, False to keep the product key around. If
        you're using deactivate to let a user move between computers it's almost always
        best to *not* erase the product key. This way you can just use
        activation_request_to_file() when the user wants to reactivate instead of forcing
        the user to re-enter their product key over-and-over again.
        """

        args = [wstr(filename)]

        if erase_p_key is True:
            args.append(1)
        else:
            args.append(0)

        self._lib.TA_DeactivationRequestToFile(self._handle, *args)

    def activate(self, extra_data=""):
        """
        Activates the product on this computer. You must call set_product_key()
        with a valid product key or have used the TurboActivate wizard sometime
        before calling this function.
        """

        if extra_data:
            options = ACTIVATE_OPTIONS(sizeof(ACTIVATE_OPTIONS()),
                                      wstr(extra_data))

            args = pointer(options)
        else:
            args = None

        self._lib.TA_Activate(self._handle, args)

    def activation_request_to_file(self, filename, extra_data=""):
        """
        Get the "activation request" file for offline activation. You must call
        set_product_key() with a valid product key or have used the TurboActivate
        Wizard sometime before calling this function.
        """
        args = [wstr(filename)]

        if extra_data:
            options = ACTIVATE_OPTIONS(sizeof(ACTIVATE_OPTIONS()),
                                      wstr(extra_data))

            args.append(pointer(options))
        else:
            args.append(None)

        self._lib.TA_ActivationRequestToFile(self._handle, *args)

    def activate_from_file(self, filename):
        """Activate from the "activation response" file for offline activation."""

        self._lib.TA_ActivateFromFile(self._handle, wstr(filename))

    def get_extra_data(self):
        """Gets the extra data you passed in using activate()"""
        buf_size = 255
        buf = wbuf(buf_size)

        try:
            self._lib.TA_GetExtraData(self._handle, buf, buf_size)

            return buf.value
        except TurboActivateFailError:
            return ""

    def is_activated(self):
        """ Checks whether the computer has been activated."""

        ret = self._lib.TA_IsActivated(self._handle)

        if ret == TA_OK:
            return True
        elif ret == TA_FAIL:
            return False

        # raise an error on all other return codes
        validate_result(ret)

    # Features

    def has_feature(self, name):
        return len(self.get_feature_value(name)) > 0

    def get_feature_value(self, name):
        """Gets the value of a feature."""
        buf_size = self._lib.TA_GetFeatureValue(self._handle, wstr(name), 0, 0)
        buf = wbuf(buf_size)

        self._lib.TA_GetFeatureValue(self._handle, wstr(name), buf, buf_size)

        return buf.value

    # Genuine

    def is_genuine(self):
        """
        Checks whether the computer is genuinely activated by verifying with the LimeLM servers.
        If reactivation is needed then it will do this as well.
        """
        ret = self._lib.TA_IsGenuine(self._handle)

        if ret == TA_OK:
            return IsGenuineResult.Genuine
        elif ret in {TA_FAIL, TA_E_REVOKED, TA_E_ACTIVATE}:
            return IsGenuineResult.NotGenuine
        elif ret == TA_E_INET:
            return IsGenuineResult.InternetError
        elif ret == TA_E_IN_VM:
            return IsGenuineResult.NotGenuineInVM
        elif ret == TA_E_FEATURES_CHANGED:
            return IsGenuineResult.GenuineFeaturesChanged

        validate_result(ret)

    # IsGenuineEx

    def is_genuine_ex(self, days_between_checks, grace_days_on_inet_err, skip_offline = False, offline_show_inet_err = False):
        """
        Checks whether the computer is genuinely activated by verifying with the LimeLM servers.
        If reactivation is needed then it will do this as well.
        """
        flags = 0

        if skip_offline:
            flags = TA_SKIP_OFFLINE

        if offline_show_inet_err:
            flags = flags | TA_OFFLINE_SHOW_INET_ERR

        options = GENUINE_OPTIONS(sizeof(GENUINE_OPTIONS()),
                                  flags,
                                  days_between_checks,
                                  grace_days_on_inet_err)

        ret = self._lib.TA_IsGenuineEx(self._handle, pointer(options))

        if ret == TA_OK:
            return IsGenuineResult.Genuine
        elif ret in {TA_FAIL, TA_E_REVOKED, TA_E_ACTIVATE}:
            return IsGenuineResult.NotGenuine
        elif ret in {TA_E_INET, TA_E_INET_DELAYED}:
            return IsGenuineResult.InternetError
        elif ret == TA_E_IN_VM:
            return IsGenuineResult.NotGenuineInVM
        elif ret == TA_E_FEATURES_CHANGED:
            return IsGenuineResult.GenuineFeaturesChanged

        validate_result(ret)

    # Trial

    def use_trial(self, verified=True, extra_data=""):
        """
        Begins the trial the first time it's called. Calling it again will validate the trial
        data hasn't been tampered with.
        """
        flags = TA_VERIFIED_TRIAL | self._flags if verified else TA_UNVERIFIED_TRIAL | self._flags

        args = [flags]

        if extra_data:
            args.append(wstr(extra_data))
        else:
            args.append(None)

        self._lib.TA_UseTrial(self._handle, *args)

    def trial_days_remaining(self, verified=True):
        """
        Get the number of trial days remaining.
        0 days if the trial has expired or has been tampered with
        (1 day means *at most* 1 day, that is it could be 30 seconds)

        You must have called "use_trial" o use this function
        """
        flags = TA_VERIFIED_TRIAL | self._flags if verified else TA_UNVERIFIED_TRIAL | self._flags
        days = c_uint32(0)

        self._lib.TA_TrialDaysRemaining(self._handle, flags, pointer(days))

        return days.value

    def extend_trial(self, extension_code, verified=True):
        """Extends the trial using a trial extension created in LimeLM."""

        flags = TA_VERIFIED_TRIAL | self._flags if verified else TA_UNVERIFIED_TRIAL | self._flags

        self._lib.TA_ExtendTrial(self._handle, flags, wstr(extension_code))

    # Utils

    def is_date_valid(self, date):
        """
        Check if the date is valid
        """

        try:
            self._lib.TA_IsDateValid(self._handle, wstr(date), TA_HAS_NOT_EXPIRED)

            return True
        except TurboActivateFlagsError as e:
            raise e
        except TurboActivateError:
            return False

    def set_custom_act_data_path(self, path):
        """
        This function allows you to set a custom folder to store the activation
        data files. For normal use we do not recommend you use this function.

        Only use this function if you absolutely must store data into a separate
        folder. For example if your application runs on a USB drive and can't write
        any files to the main disk, then you can use this function to save the activation
        data files to a directory on the USB disk.

        If you are using this function (which we only recommend for very special use-cases)
        then you must call this function on every start of your program at the very top of
        your app before any other functions are called.

        The directory you pass in must already exist. And the process using TurboActivate
        must have permission to create, write, and delete files in that directory.
        """

        self._lib.TA_SetCustomActDataPath(wstr(path))

    def set_custom_proxy(self, address):
        """
        Sets the custom proxy to be used by functions that connect to the internet.

        Proxy address in the form: http://username:password@host:port/

        Example 1 (just a host): http://127.0.0.1/
        Example 2 (host and port): http://127.0.0.1:8080/
        Example 3 (all 3): http://user:pass@127.0.0.1:8080/

        If the port is not specified, TurboActivate will default to using port 1080 for proxies.
        """
        self._lib.TA_SetCustomProxy(wstr(address))

    def get_version(self):
        """
        Gets the version number of the currently used TurboActivate library.
        This is a useful alternative for platforms which don't support file meta-data
        (like Linux, FreeBSD, and other unix variants).

        The version format is:  Major.Minor.Build.Revision
        """
        major = c_uint32(0)
        minor = c_uint32(0)
        build = c_uint32(0)
        rev = c_uint32(0)

        self._lib.TA_GetVersion(pointer(major), pointer(minor), pointer(build), pointer(rev))

        return major.value, minor.value, build.value, rev.value

    def _set_restype(self):
        self._lib.TA_PDetsFromPath.restype = validate_result
        self._lib.TA_UseTrial.restype = validate_result
        self._lib.TA_GetPKey.restype = validate_result
        self._lib.TA_IsProductKeyValid.restype = validate_result
        self._lib.TA_DeactivationRequestToFile.restype = validate_result
        self._lib.TA_Deactivate.restype = validate_result
        self._lib.TA_Activate.restype = validate_result
        self._lib.TA_ActivationRequestToFile.restype = validate_result
        self._lib.TA_ActivateFromFile.restype = validate_result
        self._lib.TA_GetExtraData.restype = validate_result
        self._lib.TA_TrialDaysRemaining.restype = validate_result
        self._lib.TA_ExtendTrial.restype = validate_result
        self._lib.TA_IsDateValid.restype = validate_result
        self._lib.TA_SetCustomProxy.restype = validate_result
        self._lib.TA_SetCustomActDataPath.restype = validate_result



# -*- coding: utf-8 -*-
#
# Copyright 2013, 2014 Develer S.r.l. (https://www.develer.com/)
# Copyright 2018 wyDay, LLC (https://wyday.com/)
#
# Current Author / maintainer:
#
#   Author: wyDay, LLC <support@wyday.com>
#
#
# Previous authors (and based on their fantastic work):
#
#   Author: Lorenzo Villani <lvillani@develer.com>
#   Author: Riccardo Ferrazzo <rferrazz@develer.com>
#
# Permission is hereby granted, free of charge, to any person obtaining a copy
# of this software and associated documentation files (the "Software"), to deal
# in the Software without restriction, including without limitation the rights
# to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
# copies of the Software, and to permit persons to whom the Software is
# furnished to do so, subject to the following conditions:
#
# The above copyright notice and this permission notice shall be included in
# all copies or substantial portions of the Software.
#
# THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
# IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
# FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
# AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
# LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING
# FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS
# IN THE SOFTWARE.

import sys
from os import path as ospath
from ctypes import (
    cdll,
    c_uint,
    c_char_p,
    c_wchar_p,
    Structure,
    create_string_buffer,
    create_unicode_buffer
)

# Utilities

# python 2.7 string.encode('utf-8') returns an str class
# python 3.6 string.encode('utf-8') returns a bytes class

is_win = sys.platform == "win32"

wbuf = create_unicode_buffer if is_win else create_string_buffer

wstr_type = c_wchar_p if is_win else c_char_p


class wstr(wstr_type):
    def __init__(self, string):
        if sys.version_info > (3, 0) and isinstance(string, str):
            super(wstr, self).__init__(string.encode('utf-8') if not is_win else string)
        else:
            super(wstr, self).__init__(string)


# Wrapper

TA_OK = 0x00000000
TA_FAIL = 0x00000001
TA_E_PKEY = 0x00000002
TA_E_ACTIVATE = 0x00000003
TA_E_INET = 0x00000004
TA_E_INUSE = 0x00000005
TA_E_REVOKED = 0x00000006
TA_E_GUID = 0x00000007
TA_E_PDETS = 0x00000008
TA_E_TRIAL = 0x00000009
TA_E_TRIAL_EUSED = 0x0000000C
TA_E_TRIAL_EEXP = 0x0000000D
TA_E_EXPIRED = 0x0000000D
TA_E_REACTIVATE = 0x0000000A
TA_E_COM = 0x0000000B
TA_E_INSUFFICIENT_BUFFER = 0x0000000E
TA_E_PERMISSION = 0x0000000F
TA_E_INVALID_FLAGS = 0x00000010
TA_E_IN_VM = 0x00000011
TA_E_EDATA_LONG = 0x00000012
TA_E_INVALID_ARGS = 0x00000013
TA_E_KEY_FOR_TURBOFLOAT = 0x00000014
TA_E_INET_DELAYED = 0x00000015
TA_E_FEATURES_CHANGED = 0x00000016
TA_E_ANDROID_NOT_INIT = 0x00000017
TA_E_NO_MORE_DEACTIVATIONS = 0x00000018
TA_E_ACCOUNT_CANCELED = 0x00000019
TA_E_ALREADY_ACTIVATED = 0x0000001A
TA_E_INVALID_HANDLE = 0x0000001B
TA_E_ENABLE_NETWORK_ADAPTERS = 0x0000001C
TA_E_ALREADY_VERIFIED_TRIAL = 0x0000001D
TA_E_TRIAL_EXPIRED = 0x0000001E
TA_E_MUST_SPECIFY_TRIAL_TYPE = 0x0000001F
TA_E_MUST_USE_TRIAL = 0x00000020
TA_E_NO_MORE_TRIALS_ALLOWED = 0x00000021
TA_E_BROKEN_WMI = 0x00000022
TA_E_INET_TIMEOUT = 0x00000023
TA_E_INET_TLS = 0x00000024

# Flags for the UseTrial() and CheckAndSavePKey() functions.

TA_SYSTEM = 0x00000001
TA_USER = 0x00000002

# Flags for the IsGeninueEx() function.

"""
If the user activated using offline activation
(ActivateRequestToFile(), ActivateFromFile() ), then with this
flag IsGenuineEx() will still try to validate with the LimeLM
servers, however instead of returning TA_E_INET (when within the
grace period) or TA_FAIL (when past the grace period) it will
instead only return TA_OK (if IsActivated()).

If you still want to get the TA_E_INET error code, without
deactivating after the grace period has expired, then use
this flag in tandem with TA_OFFLINE_SHOW_INET_ERR.

If the user activated using online activation then this flag
is ignored.
"""
TA_SKIP_OFFLINE = 0x00000001

"""
If the user activated using offline activation, and you're
using this flag in tandem with TA_SKIP_OFFLINE, then IsGenuineEx()
will return TA_E_INET on internet failure instead of TA_OK.

If the user activated using online activation then this flag
is ignored.
"""
TA_OFFLINE_SHOW_INET_ERR = 0x00000002

"""
Use the TA_DISALLOW_VM in UseTrial() to disallow trials in virtual machines.
If you use this flag in UseTrial() and the customer's machine is a Virtual
Machine, then UseTrial() will return TA_E_IN_VM.
"""
TA_DISALLOW_VM = 0x00000004

"""
Use this flag in TA_UseTrial() to tell TurboActivate to use client-side
unverified trials. For more information about verified vs. unverified trials,
see here: https://wyday.com/limelm/help/trials/
Note: unverified trials are unsecured and can be reset by malicious customers.
"""
TA_UNVERIFIED_TRIAL = 0x00000010

"""
Use the TA_VERIFIED_TRIAL flag to use verified trials instead
of unverified trials. This means the trial is locked to a particular computer.
The customer can't reset the trial.
"""
TA_VERIFIED_TRIAL = 0x00000020


# Flags for the is_date_valid() Function

TA_HAS_NOT_EXPIRED = 0x00000001


class GENUINE_OPTIONS(Structure):
    _fields_ = [
        ("nLength", c_uint),
        ("flags", c_uint),
        ("nDaysBetweenChecks", c_uint),
        ("nGraceDaysOnInetErr", c_uint),
    ]


class ACTIVATE_OPTIONS(Structure):
    _fields_ = [
        ("nLength", c_uint),
        ("sExtraData", wstr),
    ]


def load_library(path):

    if sys.platform == 'win32' or sys.platform == 'cygwin':
        return cdll.LoadLibrary(ospath.join(path, 'TurboActivate.dll'))
    elif sys.platform == 'darwin':
        return cdll.LoadLibrary(ospath.join(path, 'libTurboActivate.dylib'))

    # else: linux, bsd, etc.
    return cdll.LoadLibrary(ospath.join(path, 'libTurboActivate.so'))


def validate_result(return_code):
    # All ok, no need to perform error handling.
    if return_code == TA_OK:
        return

    # Raise an exception type appropriate for the kind of error
    if return_code == TA_FAIL:
        raise TurboActivateFailError()
    elif return_code == TA_E_PDETS:
        raise TurboActivateDatFileError()
    elif return_code == TA_E_EDATA_LONG:
        raise TurboActivateExtraDataLongError()
    elif return_code == TA_E_PKEY:
        raise TurboActivateProductKeyError()
    elif return_code == TA_E_INUSE:
        raise TurboActivateInUseError()
    elif return_code == TA_E_REVOKED:
        raise TurboActivateRevokedError()
    elif return_code == TA_E_GUID:
        raise TurboActivateGuidError()
    elif return_code == TA_E_TRIAL:
        raise TurboActivateTrialCorruptedError()
    elif return_code == TA_E_TRIAL_EUSED:
        raise TurboActivateTrialUsedError()
    elif return_code == TA_E_TRIAL_EEXP:
        raise TurboActivateTrialExpiredError()
    elif return_code == TA_E_ACTIVATE:
        raise TurboActivateNotActivatedError()
    elif return_code == TA_E_INVALID_FLAGS:
        raise TurboActivateFlagsError()
    elif return_code == TA_E_COM:
        raise TurboActivateComError()
    elif return_code == TA_E_INET:
        raise TurboActivateInetError()
    elif return_code == TA_E_PERMISSION:
        raise TurboActivatePermissionError()
    elif return_code == TA_E_NO_MORE_DEACTIVATIONS:
        raise TurboActivateNoMoreDeactivationsError()
    elif return_code == TA_E_ACCOUNT_CANCELED:
        raise TurboActivateAccountCanceledError()
    elif return_code == TA_E_INVALID_HANDLE:
        raise TurboActivateInvalidHandleError()
    elif return_code == TA_E_ALREADY_ACTIVATED:
        raise TurboActivateAlreadyActivatedError()
    elif return_code == TA_E_ENABLE_NETWORK_ADAPTERS:
        raise TurboActivateEnableNetworkAdaptersError()
    elif return_code == TA_E_ALREADY_VERIFIED_TRIAL:
        raise TurboActivateAlreadyVerifiedTrialError()
    elif return_code == TA_E_TRIAL_EXPIRED:
        raise TurboActivateTrialExpiredError()
    elif return_code == TA_E_MUST_SPECIFY_TRIAL_TYPE:
        raise TurboActivateMustSpecifyTrialTypeError()
    elif return_code == TA_E_MUST_USE_TRIAL:
        raise TurboActivateMustUseTrialError()
    elif return_code == TA_E_NO_MORE_TRIALS_ALLOWED:
        raise TurboActivateNoMoreTrialsError()
    elif return_code == TA_E_INVALID_ARGS:
        raise TurboActivateInvalidArgsError()
    elif return_code == TA_E_BROKEN_WMI:
        raise TurboActivateBrokenWMIError()
    elif return_code == TA_E_INET_TIMEOUT:
        raise TurboActivateInetTimeoutError()
    elif return_code == TA_E_INET_TLS:
        raise TurboActivateInetTLSError()

    # Otherwise bail out and raise a generic exception
    raise TurboActivateError(return_code)


#
# Exception types
#

class TurboActivateError(Exception):

    """Generic TurboActivate error"""
    pass


class TurboActivateFailError(TurboActivateError):

    """Fail error"""
    pass


class TurboActivateProductKeyError(TurboActivateError):

    """Invalid product key"""
    pass


class TurboActivateNotActivatedError(TurboActivateError):

    """The product needs to be activated."""
    pass


class TurboActivateInetError(TurboActivateError):

    """Connection to the server failed."""
    pass


class TurboActivateInetTimeoutError(TurboActivateInetError):

    """The connection to the server timed out because a long period of time
    elapsed since the last data was sent or received."""
    pass


class TurboActivateInetTLSError(TurboActivateInetError):

    """The secure connection to the activation servers failed due to a TLS or
    certificate error. More information here: https://wyday.com/limelm/help/faq/#internet-error"""
    pass


class TurboActivateInUseError(TurboActivateError):

    """The product key has already been activated with the maximum number of computers."""
    pass


class TurboActivateRevokedError(TurboActivateError):

    """The product key has been revoked."""
    pass


class TurboActivateGuidError(TurboActivateError):

    """The version GUID doesn't match that of the product details file."""
    pass


class TurboActivateTrialCorruptedError(TurboActivateError):

    """The trial data has been corrupted, using the oldest date possible."""
    pass


class TurboActivateTrialUsedError(TurboActivateError):

    """The trial extension has already been used."""
    pass


class TurboActivateTrialExpiredError(TurboActivateError):

    """
    The activation has expired or the system time has been tampered
    with. Ensure your time, timezone, and date settings are correct.
    """
    pass


class TurboActivateComError(TurboActivateError):

    """
    The hardware id couldn't be generated due to an error in the COM setup.
    Re-enable Windows Management Instrumentation (WMI) in your group policy
    editor or reset the local group policy to the default values. Contact
    your system admin for more information.

    This error is Windows only.

    This error can also be caused by the user (or another program) disabling
    the "Windows Management Instrumentation" service. Make sure the "Startup type"
    is set to Automatic and then start the service.


    To further debug WMI problems open the "Computer Management" (compmgmt.msc),
    expand the "Services and Applications", right click "WMI Control" click
    "Properties" and view the status of the WMI.
    """
    pass


class TurboActivatePermissionError(TurboActivateError):

    """
    Insufficient system permission. Either start your process as an
    admin / elevated user or call the function again with the
    TA_USER flag instead of the TA_SYSTEM flag.
    """
    pass


class TurboActivateDatFileError(TurboActivateError):

    """The product details file "TurboActivate.dat" failed to load."""
    pass


class TurboActivateFlagsError(TurboActivateError):

    """
    The flags you passed to use_trial(...) were invalid (or missing).
    """
    pass


class TurboActivateExtraDataLongError(TurboActivateError):

    """
    The "extra data" was too long. You're limited to 255 UTF-8 characters.
    Or, on Windows, a Unicode string that will convert into 255 UTF-8
    characters or less.
    """
    pass


class TurboActivateNoMoreDeactivationsError(TurboActivateError):
    """
    This product key had a limited number of allowed deactivations.
    No more deactivations are allowed for the product key. This product
    is still activated on this computer.
    """
    pass


class TurboActivateAccountCanceledError(TurboActivateError):
    """
    Can't activate or start a verified trial because the LimeLM
    account is cancelled.
    """
    pass


class TurboActivateAlreadyActivatedError(TurboActivateError):
    """
    You can't use a product key because your app is already activated
    with a product key. To use a new product key, then first deactivate using
    either the TA_Deactivate() or TA_DeactivationRequestToFile().
    """
    pass


class TurboActivateInvalidHandleError(TurboActivateError):
    """
    The handle is not valid. To get a handle use TA_GetHandle().
    """
    pass


class TurboActivateEnableNetworkAdaptersError(TurboActivateError):
    """
    There are network adapters on the system that are disabled and
    TurboActivate couldn't read their hardware properties (even after trying
    and failing to enable the adapters automatically). Enable the network adapters,
    re-run the function, and TurboActivate will be able to "remember" the adapters
    even if the adapters are disabled in the future.

    Note:   The network adapters do not need an active Internet connections. They just
            need to not be disabled. Whether they are or are not connected to the
            internet/intranet is not important and does not affect this error code at all.


    On Linux you'll get this error if you don't have any real network adapters attached.
    For example if you have no "eth[x]", "wlan[x]", "en[x]", "wl[x]", "ww[x]", or "sl[x]"
    network interface devices.

    See: https://wyday.com/limelm/help/faq/#disabled-adapters
    """
    pass


class TurboActivateAlreadyVerifiedTrialError(TurboActivateError):
    """
    The trial is already a verified trial. You need to use the "TA_VERIFIED_TRIAL"
    flag. Can't "downgrade" a verified trial to an unverified trial.
    """
    pass


class TurboActivateMustSpecifyTrialTypeError(TurboActivateError):
    """
    You must specify the trial type (TA_UNVERIFIED_TRIAL or TA_VERIFIED_TRIAL).
    And you can't use both flags. Choose one or the other. We recommend TA_VERIFIED_TRIAL.
    """
    pass


class TurboActivateMustUseTrialError(TurboActivateError):
    """
    You must call TA_UseTrial() before you can get the number of trial days remaining.
    """
    pass


class TurboActivateNoMoreTrialsError(TurboActivateError):
    """
    In the LimeLM account either the trial days is set to 0, OR the account is set
    to not auto-upgrade and thus no more verified trials can be made.
    """
    pass


class TurboActivateBrokenWMIError(TurboActivateError):
    """
    The WMI repository on the computer is broken. To fix the WMI repository
    see the instructions here:

    https://wyday.com/limelm/help/faq/#fix-broken-wmi
    """
    pass


class TurboActivateInvalidArgsError(TurboActivateError):
    """
    The arguments passed to the function are invalid. Double check your logic.
    """
    pass



#!/usr/bin/env python

import setuptools
from distutils.core import setup

setup(name="turboactivate",
      version="4.4.4",
      description="Python integration for the LimeLM TurboActivate library. This lets you add hardware-locked licensing (a.k.a. node-locked licensing) to your Python app.",
      url="https://wyday.com/limelm/help/using-turboactivate-with-python/",
      download_url="https://github.com/wyday/python-turboactivate",
      author="Develer S.r.L",
      author_email="info@develer.com",
      maintainer="wyDay, LLC",
      maintainer_email="support@wyday.com",
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Intended Audience :: Developers',
          'License :: OSI Approved :: MIT License',
          'Topic :: Software Development :: Libraries :: Python Modules',
		  'Programming Language :: Python',
		  'Programming Language :: Python :: 2',
		  'Programming Language :: Python :: 3',
      ],
      packages=["turboactivate"],
      long_description=open("README.md").read(),
      long_description_content_type="text/markdown"
)
